==> nono-0.4.4.tar.gz: nono-0.4.4/vm/vm.cpp <==
//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// VM (基本クラス)
//

#include "vm.h"
#include "debugger_private.h"
#include "mainapp.h"
#include "mainram.h"
#include "mpu.h"
#include "power.h"
#include "scheduler.h"
#include "signalthread.h"
#include "sync.h"

// グローバル参照用
VM *gVM;

// コンストラクタ
VM::VM()
{
	// SignalThread は他のスレッドより先に起動する必要があるので、先頭。
	// Scheduler は以降の他のデバイスからのイベント登録を受け付ける必要が
	// あるためその次。
	NEWDV(SignalThread, new SignalThread());
	NEWDV(Scheduler, new Scheduler());
	NEWDV(Sync, new Sync());
	NEWDV(Power, new PowerDevice());
	NEWDV(Debugger, new Debugger());

	NEWDV(Mainbus, new MainbusDevice());
	NEWDV(BusErr, new BusErrDevice());
}

// デストラクタ
VM::~VM()
{
	Dispose();
	gVM = NULL;
}

// 終了
void
VM::Dispose()
{
	// まずスレッドを止める。
	// デバッガスレッドがスケジューラスレッドにアクセスするのでデバッガが先。
	if ((bool)pDebugger) {
		gDebugger->TerminateThread();
	}
	if ((bool)pScheduler) {
		gScheduler->TerminateThread();
	}

	// 残りのスレッドを停止
	for (auto obj : gMainApp.GetObjects()) {
		auto th = dynamic_cast<ThreadDevice *>(obj);
		if (th) {
			th->TerminateThread();
		}
	}

	// 作ったときと逆順で解放する
	if ((bool)pDebugger) {
		pDebugger.reset();
	}

	if ((bool)pScheduler) {
		pScheduler.reset();
	}

	// ほかのオブジェクトの解放はグローバルデストラクタにおまかせ。
	// だめだったらまた考える。
}

// コンストラクタ後の初期化
bool
VM::Create()
{
	// d->Create() は Object を追加する場合があるため、ここで
	// ranged for は使えない。また追加されたデバイスの Create() も(念のため)
	// 実行したいため、ループ終了条件で毎回 size() を取り出して比較する。
	for (int i = 0; i < gMainApp.GetObjects().size(); i++) {
		auto dev = dynamic_cast<Device *>(gMainApp.GetObjects()[i]);
		if (dev) {
			if (!dev->Create()) {
				return false;
			}
		}
	}

	return true;
}

// 初期化
bool
VM::Init()
{
	for (auto obj : gMainApp.GetObjects()) {
		auto dev = dynamic_cast<Device *>(obj);
		if (dev) {
			if (!dev->Init()) {
				return false;
			}
		}
	}

	return true;
}

// スレッド開始
bool
VM::StartThread()
{
	for (auto obj : gMainApp.GetObjects()) {
		auto th = dynamic_cast<ThreadDevice *>(obj);
		if (th) {
			if (!th->StartThread()) {
				return false;
			}
		}
	}

	return true;
}

// 設定の適用
bool
VM::Apply()
{
	for (auto obj : gMainApp.GetObjects()) {
		auto dev = dynamic_cast<Device *>(obj);
		if (dev) {
			if (!dev->Apply()) {
				return false;
			}
		}
	}

	return true;
}

// ブートページを切り替える (共通処理)
// 派生クラスから呼ぶこと。
void
VM::SwitchBootPage(Device *parent, bool isrom)
{
	// ログ表示
	if (isrom) {
		parent->putlogf(1, lstr("SwitchBootPage ROM (Boot)"));
	} else {
		parent->putlogf(1, lstr("SwitchBootPage RAM (Normal)"));
	}

	// 同期モード指示
	gSync->RequestBootPageMode(isrom);
}

// MPU の RESET 命令によるリセット
void
VM::ResetByMPU()
{
}

==> nono-0.6.3.tar.gz: nono-0.6.3/vm/vm.cpp <==
//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// VM (基本クラス)
//

#include "vm.h"
#include "debugger.h"
#include "mainapp.h"
#include "mainbus.h"
#include "mainram.h"
#include "mpu.h"
#include "power.h"
#include "scheduler.h"
#include "signalthread.h"
#include "syncer.h"

// コンストラクタ
VM::VM()
{
	// SignalThread は他のスレッドより先に起動する必要があるので、先頭。
	// Scheduler は以降の他のデバイスからのイベント登録を受け付ける必要が
	// あるためその次。
	NEWDV(SignalThread, new SignalThread());
	NEWDV(Scheduler, new Scheduler());
	NEWDV(Syncer, new Syncer());
	NEWDV(Power, new PowerDevice());
	NEWDV(Debugger, new Debugger());
}

// デストラクタ
VM::~VM()
{
	Dispose();
}

// 終了
void
VM::Dispose()
{
	// まずスレッドを止める。
	// デバッガスレッドがスケジューラスレッドにアクセスするのでデバッガが先。
	if ((bool)pDebugger) {
		GetDebugger()->TerminateThread();
	}
	if ((bool)pScheduler) {
		GetScheduler()->TerminateThread();
	}

	// 残りのスレッドを停止
	for (auto obj : gMainApp.GetObjects()) {
		auto th = dynamic_cast<ThreadDevice *>(obj);
		if (th) {
			th->TerminateThread();
		}
	}

	// 作ったときと逆順で解放する
	if ((bool)pDebugger) {
		pDebugger.reset();
	}

	if ((bool)pScheduler) {
		pScheduler.reset();
	}

	// ほかのオブジェクトの解放はグローバルデストラクタにおまかせ。
	// だめだったらまた考える。
}

// コンストラクタ後の初期化
bool
VM::Create()
{
	// d->Create() は Object を追加する場合があるため、ここで
	// ranged for は使えない。また追加されたデバイスの Create() も
	// 実行したいため、ループ終了条件で毎回 size() を取り出して比較する。
	for (int i = 0; i < gMainApp.GetObjects().size(); i++) {
		auto dev = dynamic_cast<Device *>(gMainApp.GetObjects()[i]);
		if (dev) {
			if (!dev->Create()) {
				return false;
			}
		}
	}

	return true;
}

// 初期化
bool
VM::Init()
{
	for (auto obj : gMainApp.GetObjects()) {
		auto dev = dynamic_cast<Device *>(obj);
		if (dev) {
			if (!dev->Init()) {
				return false;
			}
		}
	}

	// デバイスの初期化が全部出来たのでメインバスの初期化。
	auto mainbus = dynamic_cast<MainbusDevice*>(pMainbus.get());
	if (mainbus->InitMainbus() == false) {
		return false;
	}

	return true;
}

// スレッド開始
bool
VM::StartThread()
{
	for (auto obj : gMainApp.GetObjects()) {
		auto th = dynamic_cast<ThreadDevice *>(obj);
		if (th) {
			if (!th->StartThread()) {
				return false;
			}
		}
	}

	return true;
}

// 設定の適用
bool
VM::Apply()
{
	for (auto obj : gMainApp.GetObjects()) {
		auto dev = dynamic_cast<Device *>(obj);
		if (dev) {
			if (!dev->Apply()) {
				return false;
			}
		}
	}

	return true;
}
